/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef CONEXUSENDPOINTSERVER_H
#define CONEXUSENDPOINTSERVER_H

#include <conexus/server.h>
#include <conexus/endpoint.h>

namespace Conexus
{

  /**
   * The EndpointServer generates Endpoints as a result of operation. This is in
   * contrast to the DataServer which generates data as a result of operation.
   *
   * The canonical example of an EndpointServer is a TCP server which as a result
   * of operation generates accepted TCP connections. Thus, the TCP server is the
   * EndpointServer, while the accepted TCP connections are DataServers.
   *
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   */
  class EndpointServer : public Server
  {

    protected:

      EndpointServer();

    public:
      typedef ConexusPointer<EndpointServer> pointer;

      virtual ~EndpointServer();

      /**
       * Starts the EndpointServer in a threaded mode. Whether a dispatcher will be used
       * is determined by the default dispatcher behavior set by Conexus::init() or
       * Conexus::set_use_dispatcher_default().
       */
      virtual void start();

      /**
       * Starts the EndpointServer in a threaded mode and explicitly determines whether a
       * dispatcher will be used, possibly overriding the default value.
       */
      virtual void start( bool use_dispatcher );

      /** Stops the EndpointServer. */
      virtual void stop();

      /**
       * Connect to this signal to receive notifications of new endpoints created by this
       * endpoint server.
       */
      sigc::signal<void, Endpoint::pointer> signal_new_endpoint();

    protected:
      sigc::signal<void, Endpoint::pointer> m_signal_new_endpoint;
      sigc::connection m_dispatcher_connection;
      std::queue<Endpoint::pointer> m_endpoint_queue;

      Glib::Mutex m_mutex_signal_endpoint;
      Glib::Mutex m_mutex_endpoint_queue;

      void queue_endpoint( Endpoint::pointer endpoint );
      void emit_endpoints( );

  };

}

#endif
