/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSEXCEPT_LISTEN_H
#define CONEXUSEXCEPT_LISTEN_H

#include <conexus/except.h>

namespace Conexus {

  /**
   * Base class for all listen exceptions.
   * @author Rick L Vinyard Jr
   * @ingroup exceptions
   */
  class listen_exception: public conexus_exception {
    public:
      listen_exception( int e = 0 ) : conexus_exception( "Unknown exception, socket not listening.", e, "listen" ) { }
      listen_exception( const std::string s, int e = 0 ) : conexus_exception( s, e, "listen" ) { }
    }
  ;

  namespace exception {


    namespace listen {

      /**
       * EADDRINUSE
       * Another socket is already listening on the same port.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class address_in_use: public listen_exception {
        public:
          address_in_use() : listen_exception( "Another socket is already listening on the same port.", EADDRINUSE ) { }
        }
      ;

      /**
       * EBADF
       * The argument s is not a valid descriptor.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class bad_fd: public listen_exception {
        public:
          bad_fd() : listen_exception( "The file descriptor is not a valid index in the descriptor table.", EBADF ) { }
        }
      ;

      /**
       * ENOTSOCK
       * The argument s is not a socket.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_socket: public listen_exception {
        public:
          not_socket() : listen_exception( "The file descriptor is not associated with a socket.", ENOTSOCK ) { }
        }
      ;

      /**
       * EOPNOTSUPP
       * The socket is not of a type that supports the listen operation.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class operation_not_supported: public listen_exception {
        public:
          operation_not_supported() : listen_exception( "The socket is not of a type that supports the listen operation.", EOPNOTSUPP ) { }
        }
      ;

      /**
       * Not bound.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_bound: public listen_exception {
        public:
          not_bound() : listen_exception( "Trying to listen on a socket that can't be bound.", -1 ) { }
        }
      ;

      }
    }
  }

#endif
