/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSFILEDESCRIPTOR_H
#define CONEXUSFILEDESCRIPTOR_H

#include <conexus/endpoint.h>

/**
 * \page manual_filedescriptor File Descriptor Endpoints
 */


namespace Conexus
{

  /**
   * This class is the top level object for all subclasses performing I/O (Input/Output)
   * operations via file descriptors (which in Linux/Unix/POSIX can be used for almost all
   * I/O).
   *
   * @author Rick L Vinyard Jr
   */
  class FileDescriptor : public Endpoint
  {
    protected:

      FileDescriptor( int fd = -1, bool close_on_destruction = true );

    public:

      typedef ConexusPointer<FileDescriptor> pointer;

      virtual ~FileDescriptor();

      /**
       * Base class method returning the raw file descriptor.
       *
       * This class is also available as the "fd" property.
       */
      virtual int fd() throw ();

      /**
       * Closes the Endpoint object by closing the member file descriptor.
       *
       * If the object is already closed will return without attempting to close.
       */
      virtual void close( bool force = false ) throw ( close_exception );

    protected:
      int m_fd;

      virtual ssize_t write_data( long int timeout, const Data data ) throw ( write_exception );

      virtual Data read_data( long int timeout, size_t s = 0 ) throw ( read_exception );

  };

}

#endif
