/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSPIPE_H
#define CONEXUSPIPE_H

#include <conexus/filedescriptor.h>

/**
 * \page manual_pipe Pipe Endpoints
 *
 * \section example_pipe Anonymous Pipe Example
 * \include anonymous_pipe.cpp
 *
 * \section example_pipe_server Anonymous Pipe Server Example
 * \include anonymous_pipe_server.cpp
 */

namespace Conexus
{

  /**
   * This class encapsulates a POSIX pipe.
   *
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   */
  class Pipe : public FileDescriptor
  {
    protected:

      Pipe( );

    public:

      typedef ConexusPointer<Pipe> pointer;

      static Pipe::pointer create( );

      virtual ~Pipe();

      virtual void open() throw ( open_exception );

      virtual void close( bool force = false ) throw ( close_exception );

      virtual void close_read( bool force = false ) throw ( close_exception );

      virtual void close_write( bool force = false ) throw ( close_exception );

      virtual ssize_t input_available();

    protected:
      int m_write_fd;

      virtual ssize_t write_data( long int timeout, const Data data ) throw ( write_exception );

      virtual Data read_data( long int timeout, size_t s = 0 ) throw ( read_exception );
  };

}

#endif
