/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSSERVER_H
#define CONEXUSSERVER_H

#include <set>
#include <queue>

#include <sys/select.h>

#include <sigc++/sigc++.h>

#include <glibmm/dispatcher.h>
#include <glibmm/thread.h>

#include <conexus/data.h>

namespace Conexus
{

  /**
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   */
  class Server
  {
    protected:

      Server();

    public:

      typedef ConexusPointer<Server> pointer;

      virtual ~Server();

      virtual void start();
      virtual void start( bool use_dispatcher );
      virtual void stop();

      /**
       * Sets the responsiveness to add_*, remove_* and stop commands in microseconds;
       * lower is more responsive; larger is less responsive; default is 500000 microseconds (or 500 ms).
       *
       * In general this refers to the timeout period of the @em select() call used internally.
       * Each cycle of the server I/O thread acquires several mutex locks at the beginning
       * of the cycle and releases them at the end of the cycle.
       *
       * This value shortens the cycle period, thereby making the server more responsive to commands at
       * the cost of increasing overhead.
         */
      virtual void set_responsiveness( long r );

      /**
       * Returns the server responsiveness level.
       */
      virtual long responsiveness();

      virtual bool is_running();

    protected:
      Glib::Thread* m_thread;
      bool m_terminate;
      bool m_running;
      Glib::Dispatcher* m_dispatcher;
      bool m_use_dispatcher;
      struct timeval m_timeval;

      virtual void service_thread_main() = 0;

    private:
      void service_thread_main_proxy();

  };

}

#endif
