/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef CONEXUS_UTILITY_H
#define CONEXUS_UTILITY_H

#include <vector>
#include <string>

namespace Conexus {

  unsigned interface_index( const std::string& name );
  std::string interface_name( unsigned ifindex );
  std::vector<std::string> interface_names();
  std::vector<std::pair<unsigned,std::string> > interfaces();
  std::vector<std::pair<std::string, std::string> > interface_ip_addresses();

  bool use_dispatcher_default();

  void set_use_dispatcher_default( bool use_dispatcher_default=true );

  /**
   * The init method should be called before any conexus servers are started.
   *
   * Generally it is a good idea to call this at the beginning of your program if you are
   * going to use the conexus servers.
   *
   * @param use_dispatcher_default If true, dispatchers will be used by default for multi-threaded
   * callbacks. For use in Gtk, Gtkmm or any other environment that needs callbacks to occur in
   * a main thread this is a good idea. Handling of the incoming data will remain multi-threaded,
   * and queued by the thread. Only data dispatching will return to the single threaded model if
   * use_dispatcher_default is true.
   */
  void init( bool use_dispatcher_default=false );

  bool initialized();

  void set_default_multicast_hops( unsigned hops );

  void set_default_multicast_loopback( bool loopback );

}

#endif
